/*****************************************************************************
 *
 * File ..................: conio.c
 * Purpose ...............: Console IO routines.
 * Last modification date : 08-Feb-2001
 *
 *****************************************************************************
 * Copyright (C) 2000-2001
 *   
 * Darryl Perry			
 * FIDO:		1:211/105
 * Internet:	dperry.geo@yahoo.com
 *
 * This file is part of Gryphon/X BBS.
 *
 * This BBS is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * Gryphon/X BBS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Gryphon/X BBS; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include <stdio.h> 
#include <unistd.h> 
#include <stropts.h> 
#include <sys/ioctl.h> 
#include <sys/types.h> 
#include <sys/poll.h> 
#include <stropts.h> 
#include <stdarg.h> 
#include <termios.h> 

#include "structs.h" 
#include "defs.h" 
#include "mci.h" 
#include "misc.h" 
#include "conio.h" 
#include "lineedit.h" 

#define UPCASE 		0
#define DNCASE 		1
#define PHONENUM 	2
#define DATE		3
#define PASSWORD	4
#define FILENAME	5
#define NUMBERS		6
#define	NAME		7
#define	FIDOAKA		8
#define	ALLGOOD		9
#define	MONEY		10
#define	UNIXNAME	11
#define	LINEEDIT	12

#define echo_on()  term_option(0)
#define echo_off() term_option(1)
#define KEY_DEL		207

char textline[100][81];
char peekch=-1;


int getkey1();
void microsleep(unsigned int microsecs);

char onekey(char *alist)
{
	int done=FALSE;
	char ch=0,ch1=0;

	while(!done){
		ch=(char)getkey(TRUE);
		if(ch>='a' && ch<='z') 
			ch-=32;
		if(strchr(alist,ch)){
			ch1=ch;
			done=TRUE;
			if(ch1!='\n')
				putch(ch1);
			}
		}
	return(ch1);
}


void putch(char chToDisplay)
{
	mci("%c",chToDisplay);
}

void send_file(char *filename)
{
	FILE *fptr;
	char ttxt[161];

	fptr=fopen(filename,"rt");
	if(fptr==NULL){
		mci("\n\rUnable to print %s\n\r",filename);
		mci("Press Any Key\n\r");
		getkey(TRUE);
		}	
	else{
		while(fgets(ttxt,161,fptr)!=NULL){
			mci(ttxt);
			}
		}
}


void clr_scr()
{
	mci("`0F\x1b[2J\x1b[1;1H");
}


void repeat(char chValue, BYTE btTimes)
{
	int l1;

	for(l1=0;l1<btTimes;l1++)
		putch(chValue);
}


int term_option(int option)
{
	struct termios attributes;

    switch(option) {
/*
**  Turn echo on
*/
	case 0:
		if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
		attributes.c_lflag |= ECHO;
		attributes.c_lflag |= ICANON;
		attributes.c_lflag |= ISIG;
		if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
		break;
/*
**  Turn echo off
*/
      case 1:
		if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
		attributes.c_lflag &= ~(ECHO);
		if(tcsetattr(STDIN_FILENO,TCSAFLUSH,&attributes) != 0) return (-1);
		break;
/*
**  Wait forever for the keyboard to be touched. (AHHHHHH!!!!!!!)
*/
      case 2:
		if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
		attributes.c_lflag    &= ~(ICANON);
		attributes.c_cc[VMIN]  = 1;
		attributes.c_cc[VTIME] = 1;
		if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
		break;

      case 3:
		if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
		attributes.c_lflag    &= ~(ICANON);
		attributes.c_lflag    &= ~(ISIG);
		attributes.c_cc[VMIN]  = 0;
		attributes.c_cc[VTIME] = 0;
		if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
		break;
/*
**  Don't be a bozo, call it with something
*/
      default:
		mci("~SMError in terminal options routine, BAD OPTION %d~SM",option);
		return(-1);
		break;
    }
	return(0);
}

void open_keyboard()
{
	term_option(1);	 
/*	term_option(2);	 */
	term_option(3);	
}

void close_keyboard()
{
	term_option(0);
}

int kbhit()
{
	int istat;
	char buf;

    istat = read(STDIN_FILENO,&buf,1);
	if(istat==1)
		peekch=buf;
	return istat;
}

int readch()
{
	char ch;
	if(peekch!=-1){
		ch=peekch;
		peekch=-1;
		}
	else{
		fflush(stdout);
		read(0,&ch,1);
		}
	return ch;
}

int getkey1()
{
	while(!kbhit()) microsleep(1000);
	fflush(stdout);
	return(readch());
}

int getkey(BOOL bWait)
{
	int istat, key,dt;
	char buf[2];
	time_t StartTime=time(NULL), TmpTm;

	fflush(stdout);
	if (0 > term_option(1)) return EOF;
	if (0 > term_option(2)) return EOF;
	if (0 > term_option(3)) return EOF; 
	do	{
      	istat = read(STDIN_FILENO,&buf,1);
		TmpTm=time(NULL);
		dt=difftime(TmpTm,StartTime);
		if(dt >= config.KbTimeOut){
			language(301); 
			term_option(0);
			if(config.ULim[thisuser.sl-1].SecondsPerDay>0)
				thisuser.tltoday-=dt;
			qbbs(TRUE);
			}
			microsleep(1500);
		} while(istat==0 && bWait);
    if (istat < 0) 	key = EOF;
	else  			key = (int)buf[0];
	term_option(0);
	if(thisuser.tltoday<1){
		language(302);	
		qbbs(TRUE);
		}
	if(config.ULim[thisuser.sl-1].SecondsPerDay>0)
		thisuser.tltoday-=dt;
	return key;
}

int get_input(char *sStr, int iMaxlen, int type)
{
	int i,ok,retval=0,done=FALSE;
	char ch=0,prevchar,dispch;

	printf("%s",sStr);
	for(i=strlen(sStr);i<iMaxlen;i++)
		printf(" ");
	for(;i>strlen(sStr);i--)
		printf("\b");
	i=strlen(sStr);
	open_keyboard();
	do{
		ok=FALSE;
		prevchar=ch;
/*		ch=getkey(TRUE); */
		fflush(stdout);
		ch=getkey1();
		dispch=ch;

		if(ch=='\b' || ch==127){
			if(i>0){
				printf("\b \b");
				i--;
				prevchar=*sStr;
				dispch='\0';
				}
			ok=FALSE;
			}
		else{
			switch(type){
				case UPCASE:
					if(ch>='a' && ch<='z') ch-=32;
					dispch=ch;
					ok=TRUE;
					break;	
				case DNCASE:
					if(ch>='A' && ch<='Z') ch+=32;
					dispch=ch;
					ok=TRUE;
					break;
				case PHONENUM:
					if(isdigit(ch) || ch=='-' || ch=='(' || ch==')') ok=TRUE;	
					break;
				case DATE:
					if(isdigit(ch) || ch=='-' || ch=='/') ok=TRUE;	
					break;
				case PASSWORD:
					dispch=254;
				case FILENAME:
					if(ch!=' ')	ok=TRUE;
					break;
				case NUMBERS:
					if(isdigit(ch))	ok=TRUE;
					break;
				case ALLGOOD:
					ok=TRUE;
					break;
				case MONEY:
					if(isdigit(ch) || ch=='.' || ch==',' || ch=='$')	ok=TRUE;
					break;
				case FIDOAKA:
					if(isdigit(ch) || ch=='/' || ch==':' || ch=='.')	ok=TRUE;
					break;
				case UNIXNAME:
					if(ch!='/' || ch!=':' || ch!='.' || ch!=' ')	ok=TRUE;
					break;
				case LINEEDIT:
					if(i==0 && ch=='/'){
						ok=FALSE;
						retval=5;
						done=TRUE;
						}
					else
						ok=TRUE;
					break;
				case NAME:
					if(isalpha(ch) || ch==' '){
						if(prevchar==' ' || i<1){
							if(ch>='a' && ch<='z'){
								ch-=32;
								}
							if(ch>='A' && ch<='Z')
								ok=TRUE;
							}
						}
					break;
				}
			if(ok){
				prevchar=sStr[i];
				if(ch!='\n' && ch!='\r' && ch!='\x1b' && ch!='\b' && ch!=127)
					sStr[i]=ch;
				else{
					sStr[i]='\0';
					}
				printf("%c",sStr[i]);
				i++;
				}	
			}
	}while(ch!=10 && i<=iMaxlen && !done);
	sStr[i]='\0';
	stripCR(sStr);
	mci("`0F");
	close_keyboard();
	return retval;
}

void get_input1(char *sStr, int iMaxlen)
{
	unsigned char	ch = 0;
	int		iPos = 0;
	int		firstchar=TRUE;


	iPos=strlen(sStr);
	mci(sStr);
	repeat(' ',iMaxlen-iPos);
	mci("\x1b[%dD",iMaxlen-iPos);

	while (ch != 10) {

		ch = getkey(TRUE);

		if(firstchar && iPos>0){
			if(ch!=27){
				repeat(' ',iMaxlen-iPos);
				mci("\x1b[%dD",iMaxlen);
				repeat(' ',iMaxlen);
				mci("\x1b[%dD",iMaxlen);
				iPos=0;
				sStr[0]='\0';
				}
			firstchar=FALSE;
			}
		else
			firstchar=FALSE;

		if ((ch == 8) || (ch == KEY_DEL) || (ch == 127)) {
			if (iPos > 0) {
				mci("\b \b");
				sStr[--iPos] = '\0';
			} else
				putch('\007');
		}

		if (ch > 31 && ch < 127) {
			if (iPos <= iMaxlen) {
				iPos++;
				sprintf(sStr, "%s%c", sStr, ch);
				mci("%c", ch);
			} else
				putch('\007');
		}
	}

	mci("`06");
}


int get_num(char *sStr)
{
	get_input(sStr,6,NUMBERS);
	return(atoi(sStr));
}

void get_name(char *sStr, int iMaxlen)         
{                                              
	unsigned char	ch = 0; 
	int		iPos = 0,iNewPos=0;


	repeat(' ',iMaxlen);
	mci("\x1b[%dD",iMaxlen);
	strcpy(sStr, "");

	while (ch != 10) {

		fflush(stdout);
		ch = getkey(TRUE);

		if ((ch == 8) || (ch == KEY_DEL) || (ch == 127)) {
			if (iPos > 0) {
				mci("\b \b");
				sStr[--iPos]='\0';
			} else 
				putch('\007');
		}

		if (ch > 31 && ch < 127) {
			if (iPos < iMaxlen) {
				iPos++;

				if (iPos == 1)
					ch = toupper(ch);

				if (ch == 32) {
					iNewPos = iPos;
					iNewPos++;
				}

				if (iNewPos == iPos)
					ch = toupper(ch);
				else
					ch = tolower(ch);

				if (iPos == 1)
					ch = toupper(ch);

				sprintf(sStr, "%s%c", sStr, ch);
				mci("%c", ch);
			} else
				putch('\007');
		}
	}

	mci("`06");
}

void get_upcase(char *sStr, int iMaxlen)         
{                                              
	get_input(sStr,iMaxlen,UPCASE);
}

void get_password(char *sStr, int iMaxlen)         
{                                              
	unsigned char	ch = 0; 
	int		iPos = 0;


	repeat(' ',iMaxlen);
	mci("\x1b[%dD",iMaxlen);
	strcpy(sStr, "");

	while (ch != 10) {
		ch = getkey(TRUE);
		if ((ch == 8) || (ch == KEY_DEL) || (ch == 127)) {
			if (iPos > 0) {
				mci("\b \b");
				sStr[--iPos]='\0';
			} else 
				putch('\007');
		}

		if (ch >= 'a' && ch <= 'z') 
			ch -=32;
		if(ch!=10){
			sprintf(sStr, "%s%c", sStr, ch);
			mci("%c", 254);
			}
	}

	mci("`08");
}

void get_phone(char *sStr)
{
	get_input(sStr,14,PHONENUM);
}

void get_date(char *sStr, int iMaxlen)
{
	get_input(sStr,10,DATE);
}

void get_aka(char *sStr)
{
	get_input(sStr,19,FIDOAKA);
}

void get_str(char *sStr, int iMaxlen)
{
	get_input(sStr,iMaxlen,ALLGOOD);
}

void get_filename(char *sStr, int iMaxlen)
{
	get_input(sStr,iMaxlen,FILENAME);
}

void input_str(char *buf,int max,unsigned char hichar,unsigned char lochar) 
{
	get_input(buf,max,ALLGOOD);
}

int editline(char *sStr, int iMaxlen)
{
	return(get_input(sStr,iMaxlen,LINEEDIT));
}

void microsleep(unsigned int microsecs)
{
	struct pollfd dummy;
	int timeout;
	if((timeout=microsecs/1000)<=0)
		timeout=1;
	poll(&dummy,0,timeout);
}
