/*****************************************************************************
 *
 * File ..................: bbs/lineedit.c
 * Purpose ...............: Message line editor.
 * Last modification date : 06-Jul-1999
 *
 *****************************************************************************
 * Copyright (C) 1997-1999
 *   
 * Michiel Broek		FIDO:		2:2801/16
 * Beekmansbos 10		Internet:	mbroek@ux123.pttnwb.nl
 * 1971 BV IJmuiden
 * the Netherlands
 *
 * This file is part of MBSE BBS.
 *
 * This BBS is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * MBSE BBS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with MBSE BBS; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include <stdio.h>
#include "structs.h"
#include "defs.h"
#include "basics.h"
#include "conio.h"
#include "mci.h"
#include "lineedit.h"

#define TEXTBUFSIZE		100

int		Line=1;
char	Message[81][TEXTBUFSIZE];


/*
 *  Internal prototypes
 */
void	Line_Edit_Append(void);		/* Append lines			     */
void	Line_Edit_Delete(void);		/* Delete lines			     */
void	Line_Edit_Edit(void);		/* Edit lines			     */
void	Line_Edit_Insert(void);		/* Insert lines			     */
void	Line_Edit_Replace(void);	/* Replace lines		     */
void	Line_Edit_Text(void);		/* Edit (replace) text in line	     */
void	Line_Edit_Center(void);		/* Center a line		     */


void Line_Edit_Append()
{
	if((Line - 1) == TEXTBUFSIZE) {
		mci("~SMMaximum message length exceeded~SM");
		return ;
	}

/*	while (TRUE) {  */
		mci("~SM");
		editline(Message[Line], 79);

		Line++;
		if((Line - 1) == TEXTBUFSIZE) {
			mci("~SMMaximum message length exceeded~SM");
			return ;
/*		} */
	}
}



void Line_Edit_Delete()
{
	int	i, start, end = 0, total;
	int	Loop;
	char	temp[81];

	while (TRUE) {
		mci("~SM~SMDelete starting at line#(1-%d): ",Line-1); 
		*temp='\0';
		start=get_num(temp);
		if((strcmp(temp, "")) == 0) {
			mci("Aborted.~SM"); 
			return;
		}
		
		if(start > (Line - 1) )
			/* Please enter a number in the range of */
			mci("~SMPlease enter a number in the range of (1 - %d)",Line-1);
		else
			break;
	}

	while (TRUE) {
		mci("Delete ending at line# (1 - %d): ", (Line - 1) );
		*temp='\0';
		end=get_num(temp);
		if((strcmp(temp, "")) == 0) {
			mci("Aborted.~SM");
			return;
		}

		if(end > (Line - 1))
			mci("~SMPlease enter a number in the range of (1 - %d)",Line-1);
		else
			break;

	}

	/* Get total by minusing the end line from the start line  */
	/* and + 1 will give you total lines between start and end */
	total = (end - start) + 1;

	/* Define loop by minusing total lines from end which will */
	/* do a loop for only the amount of lines left after the   */
	/* end line */
	Loop = Line - end++;

	/* Minus the total amount of deleted lines from the current */
	/* amount of lines to keep track of how many lines you are  */
	/* working with                                             */
	Line -= total;

	/* Do loop to copy the current message over the deleted lines */

	for (i = 0; i < Loop; i++)
		strcpy(*(Message + start++), *(Message + end++));
}



void Line_Edit_Edit()
{
	int		edit;
	char	temp[81];

	while (TRUE) {
		mci("~SMEnter line # to edit (1-%d, <CR>=Quit): ", (Line - 1) );
		*temp='\0';
		edit=get_num(temp);
		if(edit==0)
			return;

		if(edit > Line)
			mci("~SMPlease enter anumber in the range of (1-%d) ",Line-1);
		else
			break;
		}

	mci("~SMEditing Line #%d - ~SM",Line-1);
	get_str(Message[edit],79);
}



void Line_Edit_Insert()
{
	int	i,start, end = 0, total;
	char	temp[81];

	if((Line - 1) == TEXTBUFSIZE) {
		mci("~SMMaximum message length exceeded~SM");
		return;
	}

	while (TRUE) {
		mci("~SM~SMEnter line # to insert text before (1 - %d): ",Line-1);
		*temp='\0';
		start=get_num(temp);
		if(start == 0) {
			mci("Aborted.~SM"); 
			return;
			}

		if(start > (Line - 1))
			mci("~SMPlease enter a number in the range of (1 - %d)",Line-1);
		else
			break;
 		}

	total = Line - start;
	end = Line;
	Line++;
	start = Line;

	for (i = 0; i < total + 1; i++) {
		strcpy(Message[start], Message[end]);
		start--;
		end--;
	}
}



void Line_Edit_Replace()
{
	int	edit;
	char	temp[81];

	while (TRUE) {
		mci("~SM~SMEnter line # to replace (1 - %d): ",Line-1);
		*temp='\0';
		edit=get_num(temp);
		if(edit == 0)
			return;


		if(edit > Line)
		mci("~SMPlease enter a number in the range of (1 - %d) ",Line-1);
		else break;
		}

	mci("~SMLine reads: ~SM");
	mci("%-2d :%s~SM", edit,Message[edit]);
	mci("~SMEnter new text: ~SM");
	*temp='\0';
	get_str(temp, 79);
	if((strcmp(temp, "")) == 0) {
		mci("~SMUnchanged. ~SM");
	} else
		strcpy(Message[edit], temp);

	mci("~SMLine now reads: ~SM");
	mci("%-2d : %s", edit, Message[edit]);
}



void Line_Edit_Text()
{
	int	edit;
	char	temp[81];
	char	temp1[81];

	while (TRUE) {
		mci("~SM~SMEnter line # to edit (1 - %d): ", Line-1);
		*temp='\0';
		edit=get_num(temp);
		if(edit == 0)
			return;

		if(edit > Line)
		mci("~SMPlease enter a number in the range of (1 - %d) ",Line-1);
		else
			break;
		}

	mci("~SMEditing Line %d - ~SM",edit);
	mci("%s",Message[edit]);

	mci("~SMText to replace:~SM ");
	mci(":");
	*temp='\0';
	get_str(temp, 79);
	mci("Replacement text: ");
	mci(":");
	*temp1='\0';
	get_str(temp1, 79);

	strrepl(Message[edit], 81 , temp, temp1);

	mci("~SMLine now reads: ~SM");
	mci("%s~SM", edit,Message[edit]);
}



void Line_Edit_Center()
{
	int	i, j, z, center;
	int	maxlen = 78;
	char	*CEnter;
	char	temp[81];

	mci("~SM~SMEnter line # to center (1 - %d): ",Line-1);
	*temp='\0';
	center=get_num(temp);
	if(center == 0)
		return;

	CEnter = calloc(81, sizeof(char));
	center = atoi(temp);
	j = strlen(Message[center]);
	if (j >= maxlen)
		/* Line is maximum length and cannot be centered */
		mci("~SMLine is maximum length and cannot be centered~SM");
	else {
		z = 35 - (j / 2);

		for(i = 0; i < z; i++)
			strcat(CEnter," ");
		strcat(CEnter, Message[center]);
		strcpy(Message[center], CEnter);
	}

	mci("~SM%s~SM", Message[center]);
	free(CEnter);
}

void read_file(char *fname)
{
	FILE *fptr;
	char ttxt[121];

	fptr=fopen(fname,"rt");
	if(fptr==NULL) {
		return;
		}

	while(fgets(ttxt,121,fptr)){
		stripCR(ttxt);
		strcpy(Message[Line],ttxt);
		Line++;
		}
	fclose(fptr);
	
}

void save_file(char *fname)
{
	FILE *fptr;
	int i;
	
	fptr=fopen(fname,"wt");
	if(fptr==NULL) {
		mci("~SMUnable to write temp file.");
		mci("~SM%s~SM",fname);
		return;
		}
	for(i=0;i<Line;i++){
		fputs(Message[i],fptr);
		fputs("\n",fptr);
		}
	fclose(fptr);
}

void list_message()
{
	int i,cnt;
	char ttxt[91];

	for(i=1,cnt=0; i < Line; i++,cnt++) {
		sprintf(ttxt,"%3d:%s", i, Message[i]);
		mcistrset(ttxt,78);
		mci("~SM%s",ttxt);
		if(cnt>thisuser.pagelen-3){
			mci("~SM~SP");	
			cnt=0;
			}
		}
	mci("~SM");	
}



int Line_Edit(char *fname, char *quotefile)
{
	int	i, j;
	char ttxt[121];
	
	for(Line=0;Line<TEXTBUFSIZE;Line++)
		Message[Line][0]='\0';
	Line=1;

	read_file(fname);

	mci("~SC~SM`0EBegin your message now, ~UH, Type '/' on a blank line to end ");
	mci("~SMMaximum of %d lines, 80 chars per line ~SM",TEXTBUFSIZE);
	mci("`0F|...|....|....|....|....|....|....|....|....|....|....|....|....|.....|....|....`0E");
	list_message();
	while(TRUE){
		if(editline(Message[Line++],79)==5){
			if(!Line_Edit_Menu(fname)){
				return 0;
				}
			}
		}

}

int Line_Edit_Menu(char *fname)
{
	char j;
	
	while (TRUE) {
		mci("Command [`0FLSCQDITERZ?`0E]:`0F ");
		j=onekey("LSCQDITERZ?");
		switch(j){
			case '?':
				mci("~SM~SM`0EFunctions available: (Current Message: %d Lines)~SM",Line-1);
				mci("L - List message      S - Save message      C - Continue mesg~SM");
				mci("Q - Quit message      D - Delete line       I - Insert line  ~SM");
				mci("T - Text edit         E - Edit line         R - Replace line ~SM");
				mci("Z - Center line                                              ~SM");
				break;
/*			case 'C': Line_Edit_Append(); 	break; */
			case 'C': return TRUE; break;
			case 'D': Line_Edit_Delete(); 	break;
			case 'E': Line_Edit_Edit();		break;
			case 'I': Line_Edit_Insert();	break;
			case 'L': list_message();		break;
			case 'R': Line_Edit_Replace();	break;
			case 'Q': 
				mci("Quit ~SM~SM");
				mci("Are you sure [y/N] ");
				if(yesno()){
					mci("Yes~SM");
					mci("Message aborted. ~SM~SM");
					return 0;
					} 

				mci("~SMNo~SM");
				break;
			case 'T': mci("Text Edit ");	Line_Edit_Text();	break;
			case 'S': save_file(fname); 	return 0;		break;
			case 'Z': mci("Center~SM");		Line_Edit_Center();	break;
		}
	}
}


