#include <time.h>
#include <msgapi.h>

#define NUMVOTEAS           16         /* voting answers (0..max) */
#define NUMVOTEQS           20         /* voting questions (1..max) */
#define FALSE				 0
#define TRUE                 1
#define strnicmp			 strncasecmp

#define BOOL(x) (!(!(x)))
#define BitSet(arg,posn) ((arg) | (1L << (posn)))
#define BitClr(arg,posn) ((arg) & ~(1L << (posn)))
#define BitTst(arg,posn) BOOL((arg) & (1L << (posn)))
#define BitFlp(arg,posn) ((arg) ^ (1L << (posn)))


typedef char slFlags[27];
typedef unsigned char BOOL;         /* Turbo Pascal "BOOL" */
typedef unsigned char BYTE;            /* Turbo Pascal "BYTE" */
typedef signed short int integer;      /* Turbo Pascal "integer" */
typedef signed long int longint;
typedef unsigned char string;          /* Turbo Pascal "string" */
typedef unsigned int bitx;              /* shorthand for C bitx fields */
typedef enum {LOCAL, NETMAIL, ECHOMAIL, EMAIL, NEWS} MSGTYPE;
typedef enum {PUBLIC, PRIVATE} POSTTYPE;
typedef enum {DEFON,DEFOFF,ALWAYSON} SCANTYPE;

/*****************************************************************************/
/* ------------------------ TYPE DEFINITIONS ------------------------------- */
/*****************************************************************************/

typedef integer intrange[256];         /* security tables - integers */
typedef struct          /* user AC flags */
{
    bitx rlogon : 1;                    /* L - One call per day */
    bitx rchat : 1;                     /* C - Can't page the SysOp */
    bitx rnetmaildel : 1;               /* F - Force Netmail deletion */
    bitx ramsg : 1;                     /* A - Can't post AutoMessage */
    bitx rpostan : 1;                   /* * - Can't post anonymously */
    bitx rpostpvt : 1;                  /* E - Can't post private */
    bitx rpostnet : 1;                  /* N - Can't post NetMail */
    bitx rpost : 1;                     /* P - Can't post at all */
    bitx rvoting : 1;                   /* K - Can't vote */
    bitx rmsg : 1;                      /* M - Forced email deletion */
    bitx rpostecho : 1;                 /* G - Can't post EchoMail */
    bitx fnodlratio : 1;                /* 1 - No UL/DL ratio */
    bitx fnopostratio : 1;              /* 2 - No post/call ratio */
    bitx fnofilepts : 1;                /* 3 - No file points checking */
    bitx fnodeletion : 1;               /* 4 - Protection from deletion */
    bitx fnodailyratio : 1;             /* 5 - No daily DL ratio */
    bitx fnodltime : 1;                 /* 6 - No DL time check */
    bitx fnopwchange : 1;               /* 7 - No forced pw change */
    bitx ftimepercall : 1;              /* 8 - Time on per call basis */
} uflags;

typedef struct          /* USERS.DAT status flags */
{
    bitx lockedout : 1;                 /* if locked out */
    bitx udeleted : 1;                  /* if deleted */
    bitx trapactivity : 1;              /* if trapping users activity */
    bitx trapseparate : 1;              /* if trap to separate TRAP file */
    bitx chatauto : 1;                  /* if autochat trapping */
    bitx chatseparate : 1;              /* if separate chat file to trap to */
    bitx slogseparate : 1;              /* if separate SysOp log kept */
    bitx alert : 1;                     /* alert SysOp when use logs on */
} userstatus;

typedef struct          /* USERS.DAT - User flags */
{
    bitx newusermsg : 1;                /* sent newuse message? */
    bitx clsmsg : 1;                    /* clear screen before messages? */
    bitx flinput : 1;                   /* full line input? */
    bitx hotkey : 1;                    /* menu hotkeys active */
    bitx pause : 1;                     /* pause? */
    bitx novice : 1;                    /* user is at novice help level */
    bitx hiddenlog : 1;                 /* not displayed in call/online listing */
    bitx hiddenlist : 1;                /* not displayed in user listings */
} userflags;


typedef struct{
	unsigned long onn,off;
}ACSrecL;

typedef struct{
	unsigned short onn,off;
}ACSrecS;

typedef struct{
	ACSrecL mgroup,fgroup;
	ACSrecS eflags,fflags;
	unsigned long since_mn;
	unsigned int dotm, dotw, min_logons, min_nodes, tI01, tI02, tI03;
	unsigned char min_sl, gender, min_age, tC01, tC02, tC03, tC04;
}ACSrec;

typedef struct{
	char shortsig[78];
	char longsig[4][78];
}signaturerec;

typedef struct {
	char realname[37];
	char alias[37];
	char location[31];
	char street[26];
	char zipcode[11];
	char phone1[21];
	char phone2[21];
    char pwtext[21];
	longint laston, firston, pwexpire, expiredate, subdate, votedate, tltoday;
	BYTE sl;	
    char gender;                       /* gender - M/F */
    char expireto;                     /* expire to level A - Z, !=Delete */
    char comment[41];                /* SysOp comment on user */
    char notes[3][41];               
    char lockfile[9];                /* lockout msg to print */
    BYTE active;                	 /* is the user active?*/
    char birthday_mo[3];                /* user's birthday*/
    char birthday_dy[3];                /* user's birthday*/
    char birthday_yr[5];                /* user's birthday*/
	char ResChar8[2];
	slFlags	EFlags,FFlags;
    BYTE ugraphics;                    /* user graphics protocol*/
    BYTE fallback;                     /* 2nd fallback protocol*/
    BYTE linelen;                      /* line length (# cols) */
    BYTE pagelen;                      /* page length (# rows) */
    userflags flags;                   /* flags (see above) */
    userstatus status;                 /* status (see above) */
    BYTE reserved2;                    /* RESERVED */
    uflags ac;                         /* user flags */
    long userID;                    /* unique User-ID - THIS VALUE*/
    char level;                        /* current validation level */
    BYTE datefmt;                      /* Date Format*/
	BYTE clrscr;
    long macroptr;                  /* pointer for MACROS.DAT (-1 = none) */
    BYTE reserved3[17];                /* RESERVED */
    char lastgroup;                    /* last group */
    int lastmbarea;                      /* last message base # */
    int pubpost;                      /* # public posts */
    int privpost;                     /* # private posts */
    int netpost;                      /* # netmail posts */
    char email[37];                /* mailbox status*/
    long credit;                    /* NetMail points (credit) */
    long debitx;                     /* NetMail points (debitx) */
    BYTE fsetype;                      /* FSE usage (0=never,1=always,2=ask) */
    BYTE reserved4[32];                /* RESERVED */
    string qwkarc[4];                  /* QWK default archiver */
    BOOL qwkfromyou;                /* bundle from user */
    BOOL qwkbwctrl;                 /* create BW control file? */
    BOOL qwknews;                   /* include news? */
    BOOL qwkwelcome;                /* include welcome? */
    BOOL qwkgoodbye;                /* include goodbye? */
    BOOL qwknewfiles;               /* include new files list? */
    BOOL qwkpointer;                /* include pointer file? */
    BOOL qwksession;                /* include session listing? */
    BYTE reserved5[23];                /* RESERVED */
    char defprotocol;                  /* default protocol (A-Z)*/
    int lastfile;                     /* last file base # */
    int uploads;                      /* amount ULed (files) */
    int downloads;                    /* amount DLed (file) */
    int todaydl;                      /* amount DLed today (files) */
    long uk;                        /* amount ULed (kBYTEs) */
    long dk;                        /* amount DLed (kBYTEs) */
    long todaydk;                   /* amount DLed today (kBYTEs) */
    long filepoints;                /* file points remaining */
    long totaltime;                 /* total time on system */
	unsigned long int lastmsg;		  /* The last message read */
    int timebank;                     /* # mins in Time Bank */
    int totalcalls;                   /* total calls to system */
    integer ResInt1;                   /* time left today */
    integer tbtoday;                   /* bank activit +/- time */
    BYTE todaycalls;                   /* calls to system today */
    BYTE illegal;                      /* illegal logons */
    BYTE vote[NUMVOTEQS];              /* voting data */
    string language[9];                /* langauge */
    BYTE deffse;                    /* default FSE */
	signaturerec sig;
} userrec;

typedef struct {
	unsigned short int LogonsPerDay;
	signed long SecondsPerDay;
	unsigned short int DlPerDay;
	unsigned short int UlPerDay;
	unsigned short int ResInt[32];
	unsigned long DlKbPerDay;
	unsigned long UlKbPerDay;
	unsigned long ResLong[16];
	float P2CRatio;
	float UlDlRatio;
} UserLimitRec;

typedef struct {
	unsigned short zone;
	unsigned short net;
	unsigned short node;
	unsigned short point;
	char	domain[13];
}fidoaddr;

typedef struct {
	char title[3][81];
	char flags[15];
	char prompt[121];
	char ansimenu[21];
	char helpmenu[21];
	char nodemsg[76];
	short int mode, ResInt;
	char ResChar;
	char passint[9];
	int columns;
}menutoprec;

typedef struct{
	char longcmd[81];
	char shortcmd[41];
	char keys[15];
	char cmdtype[3];
	char options[61];
	ACSrec ACS;
}menucmdrec;

typedef struct{
	char mbrealname;
	char mbvisible;
	char mbansi;
	char mb8bitx;
	char mbstrip;
	char mbaddtear;
	char mbnopubstat;
	char mbnocreadit;
	char mbinternet;
	char mbfileattach;
	char mbstripcolor;	
}mbareaflags;

typedef struct{
	int idx;
	int reserved_int[16];
	char fbdisplay[61];
	char fbfile[36];
	char fbpath[81];
	char password[16];
	ACSrec ReadACS;
	ACSrec WriteACS;
	ACSrec SysopACS;
	char colors[4];
	char scantype;
	char reserved_char[184];
	unsigned long MaxFiles;
	unsigned long MaxAge;
	long reserved_long[8];
}fb_list_rec;


typedef struct{
	int idx;
	int Type;
	int Kind;
	int Active;
	int Pubpriv;
	int Allowalias;
	int scantype;
	int fidoaddr;
	int sigtype;
	int reserved_int[15];
	char mbdisplay[61];
	char mbfile[36];
	char mbpath[81];
	char password[16];
	ACSrec ReadACS;
	ACSrec WriteACS;
	ACSrec SysopACS;
	ACSrec NetworkACS;
	char OriginLine[66];
	unsigned char colors[4];
	char infofile[36];
	char reserved_char[149];
	unsigned long MaxMsgs;
	unsigned long MaxAge;
	unsigned long first;
	unsigned long last;
	unsigned long current;
	unsigned long lastread;
	unsigned long status;
	long reserved_long[3];
	mbareaflags flags;
}mb_list_rec;

typedef struct{
	char OpenSystem;
	char AskBDate;
	char AskRandPassw;
	char AskAnsi;
	char AskClrScr;
	char AskPause;
	char AskMailAddress;
	char AskHotKeys;
	char AskGender;
	char AskDataPhone;
	char AskVoicePhone;
	char AskScanPhones;
	char PageLen;
	char ScreenLen;
	char AskLang;
	char AskDateFormat;
	char AskGeoLoc;
	char AskFSE;
	char AskFLI;
}newuser_defaults_rec;

typedef struct{
	char bbsname[41];
	char location[41];
	char sysopname[41];
	char unixsysop[9];
	char ResChar1[6];
	char datapath[61];
	char menupath[61];
	char msgspath[61];
	char filepath[61];
	char textpath[61];
	char homepath[61];
	char bbspath[61];
	char nodelist[61];
	char uxpath[61];
	char dospath[61];
	char origin[61];
	char phone[81];
	char systempw[9];
	char MsgReplyHdr[101];
	char sExternalChat[100];
	char ResChar[102];
	char Sysop2Stats;
	char ForceVote;
	char NoTlBank;
	char AskLingo;
	char ChkMltiLogins;
	char RnOrAl;
	char pwinqarea[36];
	char hackwarnarea[36];
	char AllowAnsi;
	char LoginLastCall;
	char CheckVoice;
	char CheckEmail;
	char NewU_ApplyTo[37];
	char NewU_feedback;
	int newu_val;
	int iCapUserName;
	int iCrashLevel;
	int iAttachLevel;
	int iSysopArea;
	int hidesysop;
	unsigned int KbTimeOut;
	int logdays;
	int inactivebell;
	int inactivelogoff;
	int fastsysoplogon;
	int AutoMsgLock;
	int AutoMsgAnnony;
	int iExternalChat;
	int ResInt[21];
	int LogTrys;
	int PwAge;
	int LcAge;
	int LogMin;
	char startmenu[31];	
	unsigned int systemcalls;
	fidoaddr aka[20];
	mb_list_rec DefMsg;
	newuser_defaults_rec NewU;
	UserLimitRec ULim[254];
	unsigned long AutoMsgUser;
	unsigned long ResLong[7];
	struct tm *today;
	ACSrec SysopACS;
	ACSrec CoSysACS;
	ACSrec LoginACS;
}configrec;

typedef struct{
	int idx;
	char fsecmdpath[91];
	char display[61];
	ACSrec ACS;
}fserec;

typedef struct{
	char realname[37];
	char alias[37];
	char location[41];
	char nodestatus[41];	
	char homepath[41];
	char starttime[6];
	char stoptime[6];
	char phone[15];
	char bps[10];
	char sysoplogin;
	ACSrec ACS;
	int inuse;
	int idx;
}noderec;

typedef struct {
	long hdrsize;
	long recsize;
	long syssize;
	time_t lastupd;
}msgareashdr;


typedef struct          /* LASTON.DAT  - Last callers records */
{
    long caller;                    /* system caller number */
    char realname[37];               /* real name of caller */
    char alias[37];                 /* user name of caller */
    char location[31];               /* location of caller */
    long logonspeed;                /* logon speed */
    char newuser;                   /* was newuser? */
    char reserved1;                    /* RESERVED */
    char hidden;                    /* hidden from display */
    char node;                         /* node number */
    char logofftime[9];               /* logoff time */
    short uploads;                      /* number of uploads */
    short downloads;                    /* number of downloads */
    long uk;                        /* number of upload kB */
    long dk;                        /* number of download kB */
	longint laston;
    char reserved2[8];                 /* RESERVED */
}lastcallrec;

typedef struct{
	unsigned char sl;
	int active;
	char tolev;
	char description[51];
	char upgradefn[9];
	char dngradefn[9];
	int addfp;
	int addcredit;
	int addtb;
	int expireday;
	slFlags EFlags,FFlags;
}levelsrec;

typedef struct {
	char From[XMSG_FROM_SIZE];
	char To[XMSG_TO_SIZE];
	char Subj[XMSG_SUBJ_SIZE];
	char AreaName[61];
	struct _stamp date_written;
	struct _stamp date_arrived;
	char *Text;
	int colors[4];
	unsigned int replyto, replies[MAX_REPLY];
	unsigned int num_msg, cur_msg, high_msg, high_water;
	unsigned long attr;
	NETADDR orig;
	NETADDR dest;
}MsgInfoRec;

typedef struct{
	char display[31];
	int isused;
	ACSrec ACS;	
} grouprec;

typedef struct{
	grouprec grp[2];
} mfgrouprec;

struct _attributes
{
    unsigned int priv      : 1;  /* private message flag */
    unsigned int crash     : 1;  /* crash mail */
    unsigned int rcvd      : 1;  /* received by addressee */
    unsigned int sent      : 1;  /* message sent */
    unsigned int attach    : 1;  /* file attached */
    unsigned int forward   : 1;  /* message in transit */
    unsigned int orphan    : 1;  /* unknown destination */
    unsigned int killsent  : 1;  /* kill after sending */
    unsigned int local     : 1;  /* local message */
    unsigned int hold      : 1;  /* hold for pickup */
    unsigned int direct    : 1;  /* do no gating on this msg */
    unsigned int freq      : 1;  /* file request */
    unsigned int rreq      : 1;  /* return receipt requested */
    unsigned int rcpt      : 1;  /* return receipt */
    unsigned int areq      : 1;  /* audit trail request */
    unsigned int ureq      : 1;  /* update file request */

    /* extended attributes only found in @FLAGS kludge line */
    unsigned int kfs       : 1;  /* kill file when sent */
    unsigned int as        : 1;  /* archive when sent */
    unsigned int immediate : 1;  /* immediate delivery */
    unsigned int tfs       : 1;  /* truncate file when sent */
    unsigned int lock      : 1;  /* do not process this mail */
    unsigned int cfm       : 1;  /* confirm receipt request */
    unsigned int zon       : 1;  /* send mail via zone gate */
    unsigned int hub       : 1;  /* hub routing */
};




typedef struct _msg
{
    unsigned long msgnum;       /* message number (UMSGID) */
    char *reply;                /* id of message this is a reply to */
    char *msgid;                /* this messages msgid */
    char *isfrom;               /* who from */
    char *isto;                 /* who to */
    char *subj;                 /* message subject */
    char *replyarea;            /* AREA: kludge */
    char *charset_name;         /* CHRS: kludge - first string */
    int   charset_level;        /* CHRS: kludge - FTSC 0054 level number */

    unsigned int new        : 1;  /* new message? */
    unsigned int change     : 1;  /* message been changed? */
    unsigned int scanned    : 1;  /* msg been scanned? */
    unsigned int newrcvd    : 1;  /* msg just received? */
    unsigned int soteot     : 1;  /* msg sot/eot protected? */
    unsigned int invkludges : 1;  /* invalidate kludges? */
    unsigned int rawcopy    : 1;  /* copy msg text/ctrlinfo raw? */

    time_t timestamp;           /* creation date, */
    time_t time_arvd;           /* time message arrived */


    unsigned long replyto;      /* thread to previous msg */
    unsigned long replies[10];  /* thread to next msg(s) */

    struct _attributes attrib;  /* message attribute */
    int times_read;             /* times msg been read */
    int cost;                   /* cost of message */

    fidoaddr to;                 /* destination address of message */
    fidoaddr from;               /* origin address of message */
    char *text;                 /* the message buffer */
}
msg;

typedef struct{
	time_t dt;
	int logons;
	int pubmsgs;
	int privmsgs;
	int netmsgs;
	int newusers;
	int uploads;
	int dwnloads;
	int dlk;
	int minactive;
}zlogentryrec;

typedef struct{
	zlogentryrec day[365];
}zlogrec;

typedef struct{
	int idx;
	char display[31];
	char lingofile[35];
	char menupath[91];
	char textpath[91];
	ACSrec ACS;
}lingorec;
